#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <stdbool.h>
#include <ncurses.h>
#include <unistd.h>
#include <termios.h>
#include <limits.h>
#include <time.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>

#include "consts.h"
#include "base.h"

int ln_baseN(int n, int b) {
    int r = 0;
    while(n != 0) {
        n = n / b;
        r++;
    }
    return r;
}

int pw(int x, int n) {
    if (n<0)
        return 0;
    if (n==0)
        return 1;
    if (n%2==0)
        return pw(x*x, n/2);
    return x*pw(x*x, n/2);
}

int abs(int n) {
    if(n > 0) {
        return n;
    } else {
        return (-n);
    }
}

double absf(double n) {
    if(n > 0.0f) {
        return n;
    } else {
        return (-n);
    }
}

int convex_seg(int x1, int x2, double theta) {
    return (int)(((1.0f - theta) * x1 + theta * x2));
}

bool is_an_integer(char c) {
    return ((int)c >= 48 && (int)c <= 57);
}

double to_double(int n) {
    return (double)n ;
}

int to_int(double n) {
    return (int)n ;
}

double distance_pt(int x1, int x2, int y1, int y2) {
    return sqrt(to_double(pw(x2 - x1, 2) + pw(y2 - y1, 2)));
}

bool is_star_allowed(int ctype) {
    return ((CELLTYPE)ctype == BLUE) ;
}

bool is_hidden_block_allowed(int ctype) {
    return ((CELLTYPE)ctype == BLUE || (CELLTYPE)ctype == RED) ;
}

int line_count(char* filename) {
    FILE* ptr = fopen(filename, "r");
    char c = 'd';

    int n = 0 ;
    while(c != EOF) {
        if(c == '\n') {
            n += 1;
        };
        c = fgetc(ptr);
    };
    fclose(ptr);
    return (n+1);
}

int get_integer(FILE* ptr) {
    char c = fgetc(ptr);

    int res = 0 ;
    int sign = 1;

    if(c == '-') {
        sign = -1;
        c = fgetc(ptr);
    };

    while(is_an_integer(c)) {
        res = 10*res + (int)c - 48;
        c = fgetc(ptr);
    };
    return (res*sign);
}

void terminate_line(FILE* ptr) {
    char c = fgetc(ptr);
    while(c != '\n' && c != EOF) {
        c = fgetc(ptr);
    }
}